import pathlib
from typing import Annotated

from pydantic.functional_serializers import PlainSerializer

ForcePosixPath = Annotated[
    pathlib.Path,
    PlainSerializer(lambda x: x.as_posix(), return_type=str, when_used="json"),
]



import pathlib
from typing import Optional

import pytest

from competitive_verifier.documents.job import build_markdown_job
from competitive_verifier.documents.type import FrontMatter, PageRenderJob

txt_paths = set(pathlib.Path("tests/documents/data").glob("**/*.txt"))

test_build_markdown_job_params: list[
    tuple[pathlib.Path, set[pathlib.Path], Optional[PageRenderJob]]
] = [
    (
        pathlib.Path("tests/documents/data/A.md"),
        txt_paths,
        PageRenderJob(
            path=pathlib.Path("tests/documents/data/A.txt.md"),
            document_path=pathlib.Path("tests/documents/data/A.md"),
            content=b"# A",
            front_matter=FrontMatter(
                title="Test of A",
                documentation_of="tests/documents/data/A.txt",
            ),
        ),
    ),
    (
        pathlib.Path("tests/documents/data/B.md"),
        txt_paths,
        PageRenderJob(
            path=pathlib.Path("tests/documents/data/B.txt.md"),
            document_path=pathlib.Path("tests/documents/data/B.md"),
            content=b"# B",
            front_matter=FrontMatter(
                title="Test of B",
                documentation_of="tests/documents/data/B.txt",
            ),
        ),
    ),
    (
        pathlib.Path("tests/documents/data/C.md"),
        txt_paths,
        PageRenderJob(
            path=pathlib.Path("tests/documents/data/C.txt.md"),
            document_path=pathlib.Path("tests/documents/data/C.md"),
            content=b"# C",
            front_matter=FrontMatter(
                title="Test of C",
                documentation_of="tests/documents/data/C.txt",
            ),
        ),
    ),
    (
        pathlib.Path("tests/documents/data/sub/D.md"),
        txt_paths,
        PageRenderJob(
            path=pathlib.Path("tests/documents/data/D.txt.md"),
            document_path=pathlib.Path("tests/documents/data/sub/D.md"),
            content=b"# D",
            front_matter=FrontMatter(
                title="Test of D",
                documentation_of="tests/documents/data/D.txt",
            ),
        ),
    ),
    (
        pathlib.Path("tests/documents/data/sub/E.md"),
        txt_paths,
        PageRenderJob(
            path=pathlib.Path("tests/documents/data/E.txt.md"),
            document_path=pathlib.Path("tests/documents/data/sub/E.md"),
            content=b"# E",
            front_matter=FrontMatter(
                title="Test of E",
                documentation_of="tests/documents/data/E.txt",
            ),
        ),
    ),
    (
        pathlib.Path("tests/documents/data/F.md"),
        txt_paths,
        PageRenderJob(
            path=pathlib.Path("tests/documents/data/F.txt.md"),
            document_path=pathlib.Path("tests/documents/data/F.md"),
            content=b"# F",
            front_matter=FrontMatter(
                title="Test of F",
                documentation_of="tests/documents/data/F.txt",
            ),
        ),
    ),
]


@pytest.mark.parametrize(
    "path, source_paths, expected",
    test_build_markdown_job_params,
)
def test_build_markdown_job(
    path: pathlib.Path,
    source_paths: set[pathlib.Path],
    expected: Optional[PageRenderJob],
):
    assert build_markdown_job(path, source_paths=source_paths) == expected



import pytest

from competitive_verifier.documents.front_matter import (
    merge_front_matter,
    split_front_matter,
)
from competitive_verifier.documents.type import FrontMatter

test_split_FrontMatter_params: list[tuple[bytes, FrontMatter, bytes]] = [
    (
        b"""---
title: A + B
documentation_of: //examples/awk/aplusb2.awk
---
# aplusb2

String $A + B$""",
        FrontMatter(documentation_of="//examples/awk/aplusb2.awk", title="A + B"),
        b"""# aplusb2

String $A + B$""",
    ),
]


@pytest.mark.parametrize(
    "content, front_matter, markdown_content",
    test_split_FrontMatter_params,
)
def test_split_FrontMatter(
    content: bytes,
    front_matter: FrontMatter,
    markdown_content: bytes,
):
    front_matter, actual_content = split_front_matter(content)
    assert front_matter == front_matter
    assert actual_content == markdown_content


test_merge_FrontMatter_params: list[tuple[FrontMatter, bytes, bytes]] = [
    (
        FrontMatter(documentation_of="//examples/awk/aplusb2.awk", title="A + B"),
        b"""# aplusb2

String $A + B$""",
        b"""---
data: null
documentation_of: //examples/awk/aplusb2.awk
layout: null
redirect_from: null
title: A + B
---
# aplusb2

String $A + B$""",
    ),
]


@pytest.mark.parametrize(
    "front_matter, markdown_content, expected",
    test_merge_FrontMatter_params,
)
def test_merge_FrontMatter(
    front_matter: FrontMatter,
    markdown_content: bytes,
    expected: bytes,
):
    assert merge_front_matter(front_matter, markdown_content) == expected
